/*

  Copyright (c) 2009, VTT Technical Research Center of Finland
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  * Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.
  * Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
  * Neither the name of the VTT Technical Research Center of Finland nor the
  names of its contributors may be used to endorse or promote products
  derived from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY COPYRIGHT HOLDERS ''AS IS'' AND ANY
  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

/**
 * \file publisher_pk.c
 *
 * \brief A template implementation for Knowledge Processor that reads temperature values
 *  and updates them to SIB.
 *
 * Author: Jussi Kiljander, VTT Technical Research Centre of Finland
 *
 */


#include <stdio.h>
#include <string.h>
#include <kpilow/kpi_low.h>

#define MAX_SENSORS (3)  /* the max number of sensors */

typedef struct sensor
{
  ss_bnode_t bnode;
  char name[SS_OBJECT_MAX_LEN];
  char unit[SS_OBJECT_MAX_LEN];
  char location[SS_OBJECT_MAX_LEN];
  char value[SS_OBJECT_MAX_LEN];
  char prev_value[SS_OBJECT_MAX_LEN];
}sensor_t;


int read_sensor(sensor_t * sensor)
{
  char tmp_buffer[10];
  printf("Enter value for sensor %s (max 9 chars), or type exit to quit\n", sensor->name);
  scanf("%s", tmp_buffer);
  if(!strcmp("exit", tmp_buffer))
    return -1;

  strcpy(sensor->value, tmp_buffer);
  return 0;
}

void init_sensor_ontology(ss_info_t * ss_info, char * name,
                          char * location, char * unit, char * value,
                          ss_bnode_t * bnodes)
{

  ss_triple_t * triple = NULL;

  /* insert sensor to the smart space */
  ss_add_triple(&triple, ss_info->space_id, "hasSensor", name, SS_RDF_TYPE_URI, SS_RDF_TYPE_BNODE);
  ss_insert(ss_info, triple, bnodes);
  ss_delete_triples(triple);
  triple = NULL;

  ss_add_triple(&triple, bnodes->uri, "hasName", name, SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);
  ss_add_triple(&triple, bnodes->uri, "hasLocation", location, SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);
  ss_add_triple(&triple, bnodes->uri, "hasUnitType", unit, SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);
  ss_add_triple(&triple, bnodes->uri, "hasValue", value, SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);

  ss_insert(ss_info, triple, NULL);
  ss_delete_triples(triple);
}

void remove_sensor_ontology(ss_info_t * ss_info, char * uri, char * name,
                            char * location, char * unit, char * value)
{
  ss_triple_t * triples = NULL;

  ss_add_triple(&triples, ss_info->space_id, "hasSensor", uri, SS_RDF_TYPE_URI, SS_RDF_TYPE_URI);
  ss_add_triple(&triples, uri, "hasName", name, SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);
  ss_add_triple(&triples, uri, "hasLocation", location, SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);
  ss_add_triple(&triples, uri, "hasUnitType", unit, SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);
  ss_add_triple(&triples, uri, "hasValue", value, SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);

  /* remove sensor from the smart space */
  if(ss_remove(ss_info, triples) < 0)
    {
      printf("Unable to remove sensor ontology");
    }

  ss_delete_triples(triples);
}

void update_sensor_value(ss_info_t * ss_info, char * uri, char * value, char* prev_value)
{
  ss_triple_t * insert_triples = NULL;
  ss_triple_t * removed_triples = NULL;

  ss_add_triple(&insert_triples, uri, "hasValue", value, SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);
  ss_add_triple(&removed_triples, uri, "hasValue", prev_value, SS_RDF_TYPE_URI, SS_RDF_TYPE_LIT);

  ss_update(ss_info, insert_triples, removed_triples, NULL);

  ss_delete_triples(insert_triples);
  ss_delete_triples(removed_triples);

}

int main()
{
  ss_info_t ss_info;
  int i = 0;
  int inserted = 0;
  sensor_t sensors[] = {{{{0},{0}}, "Temperature", "Celsius", "Roof", {0}, {0}},
                        {{{0},{0}}, "Humidity", "Percent", "Cellar",{0}, {0}},
                        {{{0},{0}}, "Light", "Lux", "Living Room", {0}, {0}}};


   printf("Discovering new Smart Spaces..\n");
   ss_discovery(&ss_info);

   printf("Space ID = %s\n", ss_info.space_id);
 
   if(ss_join(&ss_info, "Publisher KP") != -1)
   {
     for(i = 0; i < MAX_SENSORS; i++)
     {
       if(read_sensor(&sensors[i]) < 0)
       {
         /*user typed exit */
         goto exit;
       }

      printf("Initializing sensor ontology\n");
      init_sensor_ontology(&ss_info, sensors[i].name, sensors[i].location,
                           sensors[i].unit, sensors[i].value, &sensors[i].bnode);

      /* update the previous value */
      strcpy(sensors[i].prev_value, sensors[i].value);
      inserted++;
    }

    while(1)
    {
      for(i = 0; i < MAX_SENSORS; i++)
      {
        if(read_sensor(&sensors[i]) < 0)
        {
          /*user typed exit */
          goto exit;
        }

        update_sensor_value(&ss_info, sensors[i].bnode.uri,
                           sensors[i].value, sensors[i].prev_value);

        /* update the previous value */
        strcpy(sensors[i].prev_value, sensors[i].value);

      }
    }

    exit:

    /* leaving the smart space */

    /* remove first all previous values from the smart space, so we that we do not
       leave inconsistent information */
        
    for(i = 0; i < inserted; i++)
    {
      printf("removing sensor ontology\n");
      remove_sensor_ontology(&ss_info, sensors[i].bnode.uri, sensors[i].name, sensors[i].location,
                             sensors[i].unit, sensors[i].prev_value);
    }

    ss_leave(&ss_info);

  }
  else
    printf("ERROR: Unable to join the Smart Space\n");

  return 0;
}
